@@ -1,8 +1,5 @@
 /*
- * This file is part of gtatool, a tool to manipulate Generic Tagged Arrays
- * (GTAs).
- *
- * Copyright (C) 2006, 2007, 2009, 2010, 2011
+ * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011
  * Martin Lambers <marlam@marlam.de>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,7 +24,7 @@
 #include <string>
 #include <cctype>
 #include <cstdio>
-#include <stdint.h>
+#include <cstdint>
 
 
 /**
@@ -481,29 +478,32 @@ namespace opt
         {
             std::istringstream is(argument);
             std::vector<T> vv;
-            do
+            if (argument.length() > 0)
             {
-                T v;
-                is >> v;
-                if (is.fail())
+                do
                 {
-                    return false;
-                }
-                vv.push_back(v);
-                if (!is.eof())
-                {
-                    for (size_t i = 0; i < _separator.length(); i++)
+                    T v;
+                    is >> v;
+                    if (is.fail())
                     {
-                        char c;
-                        is.get(c);
-                        if (is.fail() || c != _separator[i])
+                        return false;
+                    }
+                    vv.push_back(v);
+                    if (!is.eof())
+                    {
+                        for (size_t i = 0; i < _separator.length(); i++)
                         {
-                            return false;
+                            char c;
+                            is.get(c);
+                            if (is.fail() || c != _separator[i])
+                            {
+                                return false;
+                            }
                         }
                     }
                 }
+                while (!is.eof());
             }
-            while (!is.eof());
             if (_size_is_fixed && vv.size() != _fixed_size)
             {
                 return false;
