@@ -47,12 +47,13 @@
 #include <QLineEdit>
 #include <QRadioButton>
 #include <QTextCodec>
+#include <QCheckBox>
 
 #include <gta/gta.hpp>
 
 #include "msg.h"
 #include "opt.h"
-#include "cio.h"
+#include "fio.h"
 #include "str.h"
 #include "intcheck.h"
 
@@ -287,17 +288,17 @@ ArrayWidget::ArrayWidget(gta::header *header, QWidget *parent)
 {
     QGridLayout *layout = new QGridLayout;
     layout->addWidget(new QLabel("Dimensions:"), 0, 0, 1, 1);
-    _dimensions_label = new QLineEdit("");
-    _dimensions_label->setReadOnly(true);
-    layout->addWidget(_dimensions_label, 0, 1, 1, 3);
+    _dimensions_ledt = new QLineEdit("");
+    _dimensions_ledt->setReadOnly(true);
+    layout->addWidget(_dimensions_ledt, 0, 1, 1, 3);
     layout->addWidget(new QLabel("Components:"), 1, 0, 1, 1);
-    _components_label = new QLineEdit("");
-    _components_label->setReadOnly(true);
-    layout->addWidget(_components_label, 1, 1, 1, 3);
+    _components_ledt = new QLineEdit("");
+    _components_ledt->setReadOnly(true);
+    layout->addWidget(_components_ledt, 1, 1, 1, 3);
     layout->addWidget(new QLabel("Size:"), 2, 0, 1, 1);
-    _size_label = new QLineEdit("");
-    _size_label->setReadOnly(true);
-    layout->addWidget(_size_label, 2, 1, 1, 3);
+    _size_ledt = new QLineEdit("");
+    _size_ledt->setReadOnly(true);
+    layout->addWidget(_size_ledt, 2, 1, 1, 3);
     layout->addWidget(new QLabel("Compression:"), 3, 0, 1, 1);
     _compression_combobox = new QComboBox();
     _compression_combobox->setEditable(false);
@@ -375,7 +376,22 @@ void ArrayWidget::update()
             dimensions_string += " x ";
         }
     }
-    _dimensions_label->setText(dimensions_string.c_str());
+    if (_header->dimensions() == 0)
+    {
+        dimensions_string += "0 (empty)";
+    }
+    else if (_header->dimensions() == 1)
+    {
+        dimensions_string += " elements";
+    }
+    else
+    {
+        dimensions_string += " (";
+        dimensions_string += str::from(_header->elements());
+        dimensions_string += " elements)";
+    }
+    _dimensions_ledt->setText(dimensions_string.c_str());
+    _dimensions_ledt->setCursorPosition(0);
     std::string components_string;
     for (uintmax_t i = 0; i < _header->components(); i++)
     {
@@ -385,9 +401,19 @@ void ArrayWidget::update()
             components_string += ", ";
         }
     }
-    _components_label->setText(components_string.c_str());
-    _size_label->setText((str::from(_header->data_size()) + " bytes ("
-                + str::human_readable_memsize(_header->data_size()) + ")").c_str());
+    if (_header->components() == 0)
+    {
+        components_string += "none";
+    }
+    _components_ledt->setText(components_string.c_str());
+    _components_ledt->setCursorPosition(0);
+    std::string size_string = str::from(_header->data_size()) + " bytes";
+    if (_header->data_size() >= 1024)
+    {
+        size_string += " (" + str::human_readable_memsize(_header->data_size()) + ")";
+    }
+    _size_ledt->setText(size_string.c_str());
+    _size_ledt->setCursorPosition(0);
     while (_taglists_widget->count() > 0)
     {
         QWidget *w = _taglists_widget->widget(0);
@@ -444,7 +470,7 @@ FileWidget::~FileWidget()
 {
     if (_save_name.length() > 0 && _save_name.compare(_file_name) != 0)
     {
-        try { cio::remove(_save_name); } catch (...) {}
+        try { fio::remove(_save_name); } catch (...) {}
     }
 }
 
@@ -473,7 +499,7 @@ void FileWidget::saved_to(const std::string &save_name)
 {
     if (_save_name.length() > 0 && _save_name.compare(_file_name) != 0)
     {
-        try { cio::remove(_save_name); } catch (...) {}
+        try { fio::remove(_save_name); } catch (...) {}
     }
     _save_name = save_name;
     _is_changed = false;
@@ -539,6 +565,13 @@ GUI::GUI()
         file_import_exr_action->setEnabled(false);
     }
     file_import_menu->addAction(file_import_exr_action);
+    QAction *file_import_ffmpeg_action = new QAction(tr("Multimedia data (via FFmpeg)..."), this);
+    connect(file_import_ffmpeg_action, SIGNAL(triggered()), this, SLOT(file_import_ffmpeg()));
+    if (!cmd_is_available(cmd_find("from-ffmpeg")))
+    {
+        file_import_ffmpeg_action->setEnabled(false);
+    }
+    file_import_menu->addAction(file_import_ffmpeg_action);
     QAction *file_import_gdal_action = new QAction(tr("Remote Sensing data (via GDAL)..."), this);
     connect(file_import_gdal_action, SIGNAL(triggered()), this, SLOT(file_import_gdal()));
     if (!cmd_is_available(cmd_find("from-gdal")))
@@ -560,6 +593,13 @@ GUI::GUI()
         file_import_mat_action->setEnabled(false);
     }
     file_import_menu->addAction(file_import_mat_action);
+    QAction *file_import_pcd_action = new QAction(tr("PCD point cloud data (via PCL)..."), this);
+    connect(file_import_pcd_action, SIGNAL(triggered()), this, SLOT(file_import_pcd()));
+    if (!cmd_is_available(cmd_find("from-pcd")))
+    {
+        file_import_pcd_action->setEnabled(false);
+    }
+    file_import_menu->addAction(file_import_pcd_action);
     QAction *file_import_pfs_action = new QAction(tr("PFS floating point data (via PFS)..."), this);
     connect(file_import_pfs_action, SIGNAL(triggered()), this, SLOT(file_import_pfs()));
     if (!cmd_is_available(cmd_find("from-pfs")))
@@ -567,6 +607,13 @@ GUI::GUI()
         file_import_pfs_action->setEnabled(false);
     }
     file_import_menu->addAction(file_import_pfs_action);
+    QAction *file_import_ply_action = new QAction(tr("PLY geometry data..."), this);
+    connect(file_import_ply_action, SIGNAL(triggered()), this, SLOT(file_import_ply()));
+    if (!cmd_is_available(cmd_find("from-ply")))
+    {
+        file_import_ply_action->setEnabled(false);
+    }
+    file_import_menu->addAction(file_import_ply_action);
     QAction *file_import_rat_action = new QAction(tr("RAT RadarTools data..."), this);
     connect(file_import_rat_action, SIGNAL(triggered()), this, SLOT(file_import_rat()));
     file_import_menu->addAction(file_import_rat_action);
@@ -602,6 +649,13 @@ GUI::GUI()
         file_export_mat_action->setEnabled(false);
     }
     file_export_menu->addAction(file_export_mat_action);
+    QAction *file_export_pcd_action = new QAction(tr("PCD point cloud data (via PCL)..."), this);
+    connect(file_export_pcd_action, SIGNAL(triggered()), this, SLOT(file_export_pcd()));
+    if (!cmd_is_available(cmd_find("to-pcd")))
+    {
+        file_export_pcd_action->setEnabled(false);
+    }
+    file_export_menu->addAction(file_export_pcd_action);
     QAction *file_export_pfs_action = new QAction(tr("PFS floating point data (via PFS)..."), this);
     connect(file_export_pfs_action, SIGNAL(triggered()), this, SLOT(file_export_pfs()));
     if (!cmd_is_available(cmd_find("to-pfs")))
@@ -609,6 +663,13 @@ GUI::GUI()
         file_export_pfs_action->setEnabled(false);
     }
     file_export_menu->addAction(file_export_pfs_action);
+    QAction *file_export_ply_action = new QAction(tr("PLY geometry data..."), this);
+    connect(file_export_ply_action, SIGNAL(triggered()), this, SLOT(file_export_ply()));
+    if (!cmd_is_available(cmd_find("to-ply")))
+    {
+        file_export_ply_action->setEnabled(false);
+    }
+    file_export_menu->addAction(file_export_ply_action);
     QAction *file_export_rat_action = new QAction(tr("RAT RadarTools data..."), this);
     connect(file_export_rat_action, SIGNAL(triggered()), this, SLOT(file_export_rat()));
     file_export_menu->addAction(file_export_rat_action);
@@ -659,6 +720,9 @@ GUI::GUI()
     QAction *dimension_extract_action = new QAction(tr("&Extract dimension from current array..."), this);
     connect(dimension_extract_action, SIGNAL(triggered()), this, SLOT(dimension_extract()));
     dimension_menu->addAction(dimension_extract_action);
+    QAction *dimension_flatten_action = new QAction(tr("&Flatten dimensions of current array (make it one-dimensional)..."), this);
+    connect(dimension_flatten_action, SIGNAL(triggered()), this, SLOT(dimension_flatten()));
+    dimension_menu->addAction(dimension_flatten_action);
     QAction *dimension_merge_action = new QAction(tr("&Merge arrays from open files into new dimension..."), this);
     connect(dimension_merge_action, SIGNAL(triggered()), this, SLOT(dimension_merge()));
     dimension_menu->addAction(dimension_merge_action);
@@ -749,9 +813,9 @@ bool GUI::check_file_unchanged()
         try
         {
             QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-            FILE *fi = cio::open(fw->save_name(), "r");
+            FILE *fi = fio::open(fw->save_name(), "r");
             FILE *save_file;
-            std::string save_name = cio::mktempfile(&save_file, PACKAGE_NAME "-");
+            std::string save_name = fio::mktempfile(&save_file, PACKAGE_NAME "-");
             for (size_t i = 0; i < fw->headers().size(); i++)
             {
                 gta::header dummy_header;
@@ -759,8 +823,8 @@ bool GUI::check_file_unchanged()
                 fw->headers()[i]->write_to(save_file);
                 dummy_header.copy_data(fi, *(fw->headers()[i]), save_file);
             }
-            cio::close(save_file, save_name);
-            cio::close(fi, fw->file_name());
+            fio::close(save_file, save_name);
+            fio::close(fi, fw->file_name());
             fw->saved_to(save_name);
             QApplication::restoreOverrideCursor();
         }
@@ -868,7 +932,7 @@ QString GUI::file_save_dialog(const QString &default_suffix, const QStringList &
             FileWidget *existing_fw = reinterpret_cast<FileWidget *>(_files_widget->widget(i));
             if (existing_fw->file_name().length() > 0)
             {
-                QFileInfo existing_file_info(cio::to_sys(existing_fw->file_name()).c_str());
+                QFileInfo existing_file_info(fio::to_sys(existing_fw->file_name()).c_str());
                 if (existing_file_info.canonicalFilePath().length() > 0
                         && file_info.canonicalFilePath() == existing_file_info.canonicalFilePath())
                 {
@@ -938,7 +1002,7 @@ int GUI::run(const std::string &cmd, const std::vector<std::string> &args,
     FILE *std_err_tmp;
     try
     {
-        std_err_tmp = cio::tempfile(PACKAGE_NAME);
+        std_err_tmp = fio::tempfile(PACKAGE_NAME);
     }
     catch (std::exception &e)
     {
@@ -1007,10 +1071,10 @@ int GUI::run(const std::string &cmd, const std::vector<std::string> &args,
     /* read messages */
     try
     {
-        cio::rewind(std_err_tmp);
+        fio::rewind(std_err_tmp);
         std_err = "";
         int c;
-        while ((c = cio::getc(std_err_tmp)) != EOF)
+        while ((c = fio::getc(std_err_tmp)) != EOF)
         {
             std_err.append(1, c);
         }
@@ -1022,7 +1086,7 @@ int GUI::run(const std::string &cmd, const std::vector<std::string> &args,
     }
     try
     {
-        cio::close(std_err_tmp);
+        fio::close(std_err_tmp);
     }
     catch (...)
     {
@@ -1035,13 +1099,13 @@ void GUI::output_cmd(const std::string &cmd, const std::vector<std::string> &arg
     try
     {
         FILE *save_file;
-        std::string save_name = cio::mktempfile(&save_file, PACKAGE_NAME "-");
+        std::string save_name = fio::mktempfile(&save_file, PACKAGE_NAME "-");
         std::string std_err;
         int retval = run(cmd, args, std_err, save_file, NULL);
-        cio::close(save_file, save_name);
+        fio::close(save_file, save_name);
         if (retval != 0)
         {
-            try { cio::remove(save_name); } catch (...) {}
+            try { fio::remove(save_name); } catch (...) {}
             std::string errmsg = "<p>Command <code>";
             errmsg += cmd;
             errmsg += "</code> failed.</p>";
@@ -1076,14 +1140,14 @@ void GUI::import_from(const std::string &cmd, const std::vector<std::string> &op
         try
         {
             std::vector<std::string> args = options;
-            args.push_back(cio::to_sys(qPrintable(open_file_names[i])));
+            args.push_back(fio::to_sys(qPrintable(open_file_names[i])));
             std::string output_name(qPrintable(open_file_names[i]));
             size_t last_slash = output_name.find_last_of('/');
             size_t last_dot = output_name.find_last_of('.');
             if (last_dot != std::string::npos && (last_slash == std::string::npos || last_dot > last_slash))
             {
                 output_name.replace(last_dot, output_name.length() - last_dot, ".gta");
-                while (cio::test_e(output_name))
+                while (fio::test_e(output_name))
                 {
                     output_name.insert(last_dot, "-new");
                 }
@@ -1108,15 +1172,15 @@ void GUI::export_to(const std::string &cmd, const std::vector<std::string> &opti
         return;
     }
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    QString save_file_name = file_save_dialog(default_suffix, filters, cio::to_sys(fw->file_name()).c_str());
+    QString save_file_name = file_save_dialog(default_suffix, filters, fio::to_sys(fw->file_name()).c_str());
     if (!save_file_name.isEmpty())
     {
         try
         {
             std::string std_err;
             std::vector<std::string> args = options;
-            args.push_back(cio::to_sys(fw->save_name()));
-            args.push_back(cio::to_sys(qPrintable(save_file_name)));
+            args.push_back(fio::to_sys(fw->save_name()));
+            args.push_back(fio::to_sys(qPrintable(save_file_name)));
             int retval = run(cmd, args, std_err, NULL, NULL);
             if (retval != 0)
             {
@@ -1134,13 +1198,13 @@ void GUI::open(const std::string &file_name, const std::string &save_name)
 {
     if (file_name.length() > 0)
     {
-        QFileInfo file_info(cio::to_sys(file_name).c_str());
+        QFileInfo file_info(fio::to_sys(file_name).c_str());
         for (int i = 0; i < _files_widget->count(); i++)
         {
             FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->widget(i));
             if (fw->file_name().length() > 0)
             {
-                QFileInfo existing_file_info(cio::to_sys(fw->file_name()).c_str());
+                QFileInfo existing_file_info(fio::to_sys(fw->file_name()).c_str());
                 if (existing_file_info.canonicalFilePath().length() > 0
                         && file_info.canonicalFilePath() == existing_file_info.canonicalFilePath())
                 {
@@ -1154,8 +1218,8 @@ void GUI::open(const std::string &file_name, const std::string &save_name)
     try
     {
         const std::string &name = (save_name.length() == 0 ? file_name : save_name);
-        FILE *f = cio::open(name, "r");
-        while (cio::has_more(f, name))
+        FILE *f = fio::open(name, "r");
+        while (fio::has_more(f, name))
         {
             gta::header *hdr = new gta::header;
             hdr->read_from(f);
@@ -1164,14 +1228,14 @@ void GUI::open(const std::string &file_name, const std::string &save_name)
         }
         if (headers.size() == 0)
         {
-            cio::close(f, name);
+            fio::close(f, name);
             QMessageBox::critical(this, "Error", "File is empty");
         }
         else
         {
             FileWidget *fw = new FileWidget(file_name, save_name, headers);
             connect(fw, SIGNAL(changed(const std::string &, const std::string &)), this, SLOT(file_changed(const std::string &, const std::string &)));
-            _files_widget->addTab(fw, (file_name.length() == 0 ? "(unnamed)" : QString(cio::to_sys(cio::basename(file_name)).c_str())));
+            _files_widget->addTab(fw, (file_name.length() == 0 ? "(unnamed)" : QString(fio::to_sys(fio::basename(file_name)).c_str())));
             _files_widget->tabBar()->setTabTextColor(_files_widget->indexOf(fw), (fw->is_saved() ? "black" : "red"));
             _files_widget->setCurrentWidget(fw);
         }
@@ -1214,8 +1278,8 @@ void GUI::file_save()
     try
     {
         QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-        FILE *fi = cio::open(fw->save_name(), "r");
-        FILE *fo = cio::open(fw->file_name() + ".tmp", "w");
+        FILE *fi = fio::open(fw->save_name(), "r");
+        FILE *fo = fio::open(fw->file_name() + ".tmp", "w");
         for (size_t i = 0; i < fw->headers().size(); i++)
         {
             gta::header dummy_header;
@@ -1226,13 +1290,13 @@ void GUI::file_save()
         /* This is a stupid and unsafe way to switch to the new file, but it works
          * cross-platform and also over NFS etc: after this, the file exists and
          * has the expected contents. */
-        cio::close(fo, fw->file_name() + ".tmp");
-        cio::close(fi, fw->file_name());
-        try { cio::remove(fw->file_name()); } catch (...) { }
-        cio::rename(fw->file_name() + ".tmp", fw->file_name());
+        fio::close(fo, fw->file_name() + ".tmp");
+        fio::close(fi, fw->file_name());
+        try { fio::remove(fw->file_name()); } catch (...) { }
+        fio::rename(fw->file_name() + ".tmp", fw->file_name());
         fw->saved_to(fw->file_name());
         _files_widget->tabBar()->setTabTextColor(_files_widget->indexOf(fw), "black");
-        _files_widget->tabBar()->setTabText(_files_widget->indexOf(fw), cio::basename(fw->file_name()).c_str());
+        _files_widget->tabBar()->setTabText(_files_widget->indexOf(fw), fio::basename(fw->file_name()).c_str());
         QApplication::restoreOverrideCursor();
     }
     catch (std::exception &e)
@@ -1252,7 +1316,7 @@ void GUI::file_save_as()
     QString file_name = file_save_dialog();
     if (!file_name.isEmpty())
     {
-        fw->set_file_name(cio::from_sys(qPrintable(file_name)));
+        fw->set_file_name(fio::from_sys(qPrintable(file_name)));
         file_save();
     }
 }
@@ -1326,6 +1390,11 @@ void GUI::file_import_exr()
     import_from("from-exr", std::vector<std::string>(), QStringList("EXR files (*.exr)"));
 }
 
+void GUI::file_import_ffmpeg()
+{
+    import_from("from-ffmpeg", std::vector<std::string>(), QStringList("Any multimedia files (*.*)"));
+}
+
 void GUI::file_import_gdal()
 {
     import_from("from-gdal", std::vector<std::string>(), QStringList("TIFF files (*.tif *.tiff)"));
@@ -1341,11 +1410,21 @@ void GUI::file_import_mat()
     import_from("from-mat", std::vector<std::string>(), QStringList("MATLAB files (*.mat)"));
 }
 
+void GUI::file_import_pcd()
+{
+    import_from("from-pcd", std::vector<std::string>(), QStringList("PCD files (*.pcd)"));
+}
+
 void GUI::file_import_pfs()
 {
     import_from("from-pfs", std::vector<std::string>(), QStringList("PFS files (*.pfs)"));
 }
 
+void GUI::file_import_ply()
+{
+    import_from("from-ply", std::vector<std::string>(), QStringList("PLY files (*.ply)"));
+}
+
 void GUI::file_import_rat()
 {
     import_from("from-rat", std::vector<std::string>(), QStringList("RAT RadarTools files (*.rat)"));
@@ -1419,11 +1498,21 @@ void GUI::file_export_rat()
     export_to("to-rat", std::vector<std::string>(), "rat", QStringList("RAT RadarTools files (*.rat)"));
 }
 
+void GUI::file_export_pcd()
+{
+    export_to("to-pcd", std::vector<std::string>(), "pcd", QStringList("PCD files (*.pcd)"));
+}
+
 void GUI::file_export_pfs()
 {
     export_to("to-pfs", std::vector<std::string>(), "pfs", QStringList("PFS files (*.pfs)"));
 }
 
+void GUI::file_export_ply()
+{
+    export_to("to-ply", std::vector<std::string>(), "ply", QStringList("PLY files (*.ply)"));
+}
+
 void GUI::file_export_raw()
 {
     if (!check_have_file())
@@ -1467,7 +1556,7 @@ void GUI::stream_merge()
     for (int i = 0; i < _files_widget->count(); i++)
     {
         FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->widget(i));
-        args.push_back(cio::to_sys(fw->save_name()));
+        args.push_back(fio::to_sys(fw->save_name()));
     }
     output_cmd("stream-merge", args, "");
 }
@@ -1495,8 +1584,8 @@ void GUI::stream_split()
         _last_file_save_as_dir = file_dialog->directory();
         FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
         std::vector<std::string> args;
-        args.push_back(cio::to_sys(fw->save_name()));
-        args.push_back(cio::to_sys(std::string(qPrintable(QDir(dir_name).canonicalPath())) + "/%9N.gta"));
+        args.push_back(fio::to_sys(fw->save_name()));
+        args.push_back(fio::to_sys(std::string(qPrintable(QDir(dir_name).canonicalPath())) + "/%9N.gta"));
         std::string std_err;
         int retval = run("stream-split", args, std_err, NULL, NULL);
         if (retval != 0)
@@ -1515,7 +1604,7 @@ void GUI::stream_extract()
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
     int index = fw->arrays_widget()->currentIndex();
     std::vector<std::string> args;
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     args.push_back(str::from(index));
     output_cmd("stream-extract", args, "");
 }
@@ -1590,7 +1679,7 @@ void GUI::array_extract()
     args.push_back("-h");
     args.push_back(qPrintable(high_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("extract", args, "");
 }
 
@@ -1633,7 +1722,7 @@ void GUI::array_fill()
     args.push_back("-v");
     args.push_back(qPrintable(val_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("fill", args, "");
 }
 
@@ -1668,7 +1757,7 @@ void GUI::array_merge()
     for (int i = 0; i < _files_widget->count(); i++)
     {
         FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->widget(i));
-        args.push_back(cio::to_sys(fw->save_name()));
+        args.push_back(fio::to_sys(fw->save_name()));
     }
     output_cmd("merge", args, "");
 }
@@ -1703,7 +1792,7 @@ void GUI::array_resize()
     args.push_back("-d");
     args.push_back(qPrintable(dim_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("resize", args, "");
 }
 
@@ -1749,7 +1838,7 @@ void GUI::array_set()
     args.push_back("-i");
     args.push_back(qPrintable(indices_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("set", args, "");
 }
 
@@ -1783,7 +1872,7 @@ void GUI::dimension_add()
     args.push_back("-d");
     args.push_back(qPrintable(dim_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("dimension-add", args, "");
 }
 
@@ -1823,10 +1912,44 @@ void GUI::dimension_extract()
     args.push_back("-i");
     args.push_back(qPrintable(index_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("dimension-extract", args, "");
 }
 
+void GUI::dimension_flatten()
+{
+    if (!check_have_file() || !check_file_unchanged())
+    {
+        return;
+    }
+    QDialog *dialog = new QDialog(this);
+    dialog->setModal(true);
+    dialog->setWindowTitle("Flatten dimensions (make one-dimensional)");
+    QGridLayout *layout = new QGridLayout;
+    QCheckBox *p_checkbox = new QCheckBox("Prepend original coordinates to each array element");
+    layout->addWidget(p_checkbox, 0, 0, 1, 2);
+    QPushButton *ok_btn = new QPushButton(tr("&OK"));
+    ok_btn->setDefault(true);
+    connect(ok_btn, SIGNAL(clicked()), dialog, SLOT(accept()));
+    layout->addWidget(ok_btn, 1, 0);
+    QPushButton *cancel_btn = new QPushButton(tr("&Cancel"), dialog);
+    connect(cancel_btn, SIGNAL(clicked()), dialog, SLOT(reject()));
+    layout->addWidget(cancel_btn, 1, 1);
+    dialog->setLayout(layout);
+    if (dialog->exec() == QDialog::Rejected)
+    {
+        return;
+    }
+    std::vector<std::string> args;
+    if (p_checkbox->isChecked())
+    {
+        args.push_back("-p");
+    }
+    FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
+    args.push_back(fio::to_sys(fw->save_name()));
+    output_cmd("dimension-flatten", args, "");
+}
+
 void GUI::dimension_merge()
 {
     if (!check_have_file() || !check_all_files_unchanged())
@@ -1837,7 +1960,7 @@ void GUI::dimension_merge()
     for (int i = 0; i < _files_widget->count(); i++)
     {
         FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->widget(i));
-        args.push_back(cio::to_sys(fw->save_name()));
+        args.push_back(fio::to_sys(fw->save_name()));
     }
     output_cmd("dimension-merge", args, "");
 }
@@ -1872,7 +1995,7 @@ void GUI::dimension_reorder()
     args.push_back("-i");
     args.push_back(qPrintable(indices_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("dimension-reorder", args, "");
 }
 
@@ -1906,7 +2029,7 @@ void GUI::dimension_reverse()
     args.push_back("-i");
     args.push_back(qPrintable(indices_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("dimension-reverse", args, "");
 }
 
@@ -1940,7 +2063,7 @@ void GUI::dimension_split()
     args.push_back("-d");
     args.push_back(qPrintable(dim_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("dimension-split", args, "");
 }
 
@@ -1982,7 +2105,7 @@ void GUI::component_add()
     args.push_back("-i");
     args.push_back(qPrintable(index_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("component-add", args, "");
 }
 
@@ -2040,7 +2163,7 @@ void GUI::component_compute()
         args.push_back(qPrintable(expressions[i]));
     }
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("component-compute", args, "");
 }
 
@@ -2060,23 +2183,29 @@ void GUI::component_convert()
     layout->addWidget(comp_label, 0, 0, 1, 2);
     QLineEdit *comp_edit = new QLineEdit("");
     layout->addWidget(comp_edit, 1, 0, 1, 2);
+    QCheckBox *n_checkbox = new QCheckBox("Normalize integers when converting to floating point");
+    layout->addWidget(n_checkbox, 2, 0, 1, 2);
     QPushButton *ok_btn = new QPushButton(tr("&OK"));
     ok_btn->setDefault(true);
     connect(ok_btn, SIGNAL(clicked()), dialog, SLOT(accept()));
-    layout->addWidget(ok_btn, 2, 0);
+    layout->addWidget(ok_btn, 3, 0);
     QPushButton *cancel_btn = new QPushButton(tr("&Cancel"), dialog);
     connect(cancel_btn, SIGNAL(clicked()), dialog, SLOT(reject()));
-    layout->addWidget(cancel_btn, 2, 1);
+    layout->addWidget(cancel_btn, 3, 1);
     dialog->setLayout(layout);
     if (dialog->exec() == QDialog::Rejected)
     {
         return;
     }
     std::vector<std::string> args;
+    if (n_checkbox->isChecked())
+    {
+        args.push_back("-n");
+    }
     args.push_back("-c");
     args.push_back(qPrintable(comp_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("component-convert", args, "");
 }
 
@@ -2110,7 +2239,7 @@ void GUI::component_extract()
     args.push_back("-k");
     args.push_back(qPrintable(index_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("component-extract", args, "");
 }
 
@@ -2124,7 +2253,7 @@ void GUI::component_merge()
     for (int i = 0; i < _files_widget->count(); i++)
     {
         FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->widget(i));
-        args.push_back(cio::to_sys(fw->save_name()));
+        args.push_back(fio::to_sys(fw->save_name()));
     }
     output_cmd("component-merge", args, "");
 }
@@ -2159,7 +2288,7 @@ void GUI::component_reorder()
     args.push_back("-i");
     args.push_back(qPrintable(indices_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("component-reorder", args, "");
 }
 
@@ -2199,7 +2328,7 @@ void GUI::component_set()
     args.push_back("-v");
     args.push_back(qPrintable(values_edit->text().simplified().replace(' ', "")));
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("component-set", args, "");
 }
 
@@ -2211,7 +2340,7 @@ void GUI::component_split()
     }
     std::vector<std::string> args;
     FileWidget *fw = reinterpret_cast<FileWidget *>(_files_widget->currentWidget());
-    args.push_back(cio::to_sys(fw->save_name()));
+    args.push_back(fio::to_sys(fw->save_name()));
     output_cmd("component-split", args, "");
 }
 
@@ -2277,7 +2406,7 @@ extern "C" int gtatool_gui(int argc, char *argv[])
         gui->show();
         for (size_t i = 0; i < arguments.size(); i++)
         {
-            gui->open(cio::from_sys(arguments[i]), cio::from_sys(arguments[i]));
+            gui->open(fio::from_sys(arguments[i]), fio::from_sys(arguments[i]));
         }
         retval = app->exec();
         delete gui;
