@@ -27,7 +27,7 @@
 #include <cstddef>
 
 #include "str.h"
-#include "cio.h"
+#include "fio.h"
 #include "msg.h"
 #include "intcheck.h"
 #include "endianness.h"
@@ -194,6 +194,12 @@ void type_from_string(const std::string &s, gta::type *t, uintmax_t *size)
 
 void typelist_from_string(const std::string &s, std::vector<gta::type> *types, std::vector<uintmax_t> *sizes)
 {
+    types->clear();
+    sizes->clear();
+    if (s.empty())
+    {
+        return;
+    }
     size_t i = 0;
     do
     {
@@ -361,6 +367,10 @@ void value_from_string(const std::string &s, const gta::type t, const uintmax_t
 void valuelist_from_string(const std::string &s, const std::vector<gta::type> &types,
         const std::vector<uintmax_t> &sizes, void *valuelist)
 {
+    if ((types.size() == 0 && !s.empty()) || (types.size() > 0 && s.empty()))
+    {
+        throw exc("invalid number of values");
+    }
     size_t i = 0;
     size_t blob_index = 0;
     char *value = static_cast<char *>(valuelist);
@@ -546,19 +556,19 @@ void element_loop_t::start(
     _buf.resize(0);
 }
 
-void *element_loop_t::read()
+void *element_loop_t::read(size_t n)
 {
-    if (_buf.size() == 0)
+    if (_buf.size() < checked_cast<size_t>(n * _header_in.element_size()))
     {
-        _buf.resize(checked_cast<size_t>(_header_in.element_size()));
+        _buf.resize(n * _header_in.element_size());
     }
-    _header_in.read_elements(_state_in, _file_in, 1, _buf.ptr());
+    _header_in.read_elements(_state_in, _file_in, n, _buf.ptr());
     return _buf.ptr();
 }
 
-void element_loop_t::write(const void *element)
+void element_loop_t::write(const void *element, size_t n)
 {
-    _header_out.write_elements(_state_out, _file_out, 1, element);
+    _header_out.write_elements(_state_out, _file_out, n, element);
 }
 
 const std::string array_loop_t::_stdin_name = "standard input";
@@ -579,7 +589,7 @@ array_loop_t::~array_loop_t()
     {
         try
         {
-            cio::close(_file_in, filename_in());
+            fio::close(_file_in, filename_in());
         }
         catch (...)
         {
@@ -589,7 +599,7 @@ array_loop_t::~array_loop_t()
     {
         try
         {
-            cio::close(_file_out, filename_out());
+            fio::close(_file_out, filename_out());
         }
         catch (...)
         {
@@ -616,7 +626,7 @@ void array_loop_t::start(const std::vector<std::string> &filenames_in,
     }
     else
     {
-        _file_in = cio::open(_filenames_in[0], "r");
+        _file_in = fio::open(_filenames_in[0], "r");
     }
     if (_filename_out.length() == 0)
     {
@@ -624,7 +634,7 @@ void array_loop_t::start(const std::vector<std::string> &filenames_in,
     }
     else
     {
-        _file_out = cio::open(_filename_out, "w");
+        _file_out = fio::open(_filename_out, "w");
     }
 }
 
@@ -635,14 +645,14 @@ void array_loop_t::finish()
         FILE *f = _file_out;
         const std::string &name = filename_out();
         _file_out = NULL;
-        cio::close(f, name);
+        fio::close(f, name);
     }
     if (_file_in && _file_in != gtatool_stdin)
     {
         FILE *f = _file_in;
         const std::string &name = filename_in();
         _file_in = NULL;
-        cio::close(f, name);
+        fio::close(f, name);
     }
 }
 
@@ -658,7 +668,7 @@ const std::string &array_loop_t::filename_out() throw ()
 
 bool array_loop_t::read(gta::header &header_in, std::string &name_in)
 {
-    while (!cio::has_more(_file_in))
+    while (!fio::has_more(_file_in))
     {
         if (_filenames_in.size() == 0)
         {
@@ -668,13 +678,13 @@ bool array_loop_t::read(gta::header &header_in, std::string &name_in)
         {
             FILE *f = _file_in;
             _file_in = NULL;
-            cio::close(f, filename_in());
+            fio::close(f, filename_in());
             if (_filename_index + 1 == _filenames_in.size())
             {
                 return false;
             }
             _filename_index++;
-            _file_in = cio::open(_filenames_in.at(_filename_index), "r");
+            _file_in = fio::open(_filenames_in.at(_filename_index), "r");
             _file_index_in = 0;
         }
     }
@@ -695,7 +705,7 @@ bool array_loop_t::read(gta::header &header_in, std::string &name_in)
 
 void array_loop_t::write(const gta::header &header_out, std::string &name_out)
 {
-    if (cio::isatty(_file_out))
+    if (fio::isatty(_file_out))
     {
         throw exc("refusing to write to a tty");
     }
